<p class="success"><i class="icon-checked"></i>You are connected to <b>{cluster_name}</b></p>
<p><b>The following variables were instantiated</b></p>
<ul class="variables">
    <li>sc = <a href="https://spark.apache.org/docs/latest/api/python/reference/api/pyspark.SparkContext.html"
                       target="_blank">SparkContext</a></li>
    <li>spark = <a
            href="https://spark.apache.org/docs/latest/api/python/reference/pyspark.sql/api/pyspark.sql.SparkSession.html"
            target="_blank">SparkSession</a></li>
</ul>
<p><b>Connection details</b></p>
<ul class="variables">
    <li>Spark version is {spark_version}</li>
    <li class="success-history-text">Spark WebUI is not available</li>
    <li class="success-metrics-text">Spark Metrics Dashboard is not activated (see config bundles)</li>
    <li>Spark driver logs of the running application <a class="success-show-logs-action"></a></li>
</ul>
<div class="log-connected">
    <pre class="logs">Waiting for the remaining job to finish to retrieve the logs..</pre>
</div>