<p>You are going to connect to:<br><span class="cluster-name">{cluster_name}</span></p><p>&nbsp;</p>
<p>You can configure the following <a href="https://spark.apache.org/docs/latest/configuration#available-properties"
                                      target="_blank" title="SparkConf options">options</a>.
<br>Environment variables can be used via {ENV_VAR_NAME}.</p><br>

<div class="new-option">
</div>

<div id="bundled-options">
    <br><label>Bundled configurations</label><br>
    <p>These options will be overwritten by non-bundled options if specified</p>
</div>

<p>&nbsp;</p>
<h2>Selected configuration</h2>

<ul class="spark-options">

</ul>